use crate::stack_vec::StackVec;
use bitvec::prelude::*;

pub(crate) fn unstuff_bits<const MAX_BYTES: usize>(
    mut bits: BitArray<[u8; MAX_BYTES], Msb0>,
    len: usize,
) -> Option<StackVec<MAX_BYTES, u8>> {
    let mut one_count = 0;
    let mut offset = 0;
    for i in 0..len {
        if one_count == 5 {
            // bit stuffing is expected
            if bits[i] {
                // we expected a zero - this data is invalid. abort
                return None;
            }

            offset += 1;
            one_count = 0;

            continue;
        }

        if bits[i] {
            one_count += 1;
        } else {
            one_count = 0;
        }

        let val = bits[i];
        bits.set(i - offset, val);
    }

    let actual_len = len - offset;
    if actual_len % 8 != 0 {
        // we have some extra bits - abort
        return None;
    }

    let mut out = StackVec::new();

    for i in 0..(actual_len / 8) {
        out.push(bits.as_raw_slice()[i]).unwrap();
    }

    Some(out)
}
